! ******************************************************************
! ******************************************************************

subroutine dinamicsubg(n,m,x,lb,ub,equatn,linear,jcnnzmax,hnnzmax,erropt,&
                       outiter,l,time,inform)                       
                 
use myproblem, only: evalf,evals,evalc

	implicit none
	
	! SCALARS ARGUMENTS
	integer,intent(in) :: n,m,jcnnzmax,hnnzmax,erropt
	integer,intent(out) :: outiter,l,inform
	real(kind=4),intent(out) :: time
	
	! ARRAYS ARGUMENTS
	real(kind=8),intent(inout) :: x(n)
	real(kind=8),intent(in) :: lb(n),ub(n)
	logical,intent(in) :: equatn(m),linear(m)
	
	! LOCAL SCALARS
	integer :: i,lprev,nfev,ngev,maxoutiter,itCG,infoCG,flag
	real(kind=8), parameter :: zero = 0.0d0
	real(kind=8) :: f,frec,freckl,flev,delta,sigma,R,Rconst,beta,t,    &
	                norm2s,opttol,optconv,optc,theta,lambda,gama,csupn,nu
	real(kind=4) :: timeini,timefin
	logical :: iprint
	
	! LOCAL ARRAYS
	real(kind=8) :: w(n),s(n),xrec(n),c(m)


	! LOCAL ARRAYS
	
	!---------------------------------------------------------------------   
	!     Initialization
	!---------------------------------------------------------------------
	
	! Start timing
		
	call cpu_time(timeini)
	
	! Print problem information?	
	
	iprint = .true.	
		
	if ( iprint ) then
			write(*,1000) n
	end if
	
	! Check the feasibility of x 

	csupn = 0.0d0
	do i = 1,n
		csupn = max( csupn, x(i)  - ub(i) )
		csupn = max( csupn, lb(i) - x(i) )
	end do
	
	if ( csupn > 0.0d0 ) then
		inform = - 1
		write(*,1001)
		return
	end if

	do i = 1,m
		call evalc(n,x,i,c(i),flag)
	end do

	do i = 1,m
		if ( equatn(i) ) then
			csupn = max( csupn, abs( c(i) ) )
		else
			csupn = max( csupn, c(i) )
		end if
	end do
	
	if ( csupn > 1.0d-8 ) then
		inform = - 1
		write(*,1001) 
		write(*,*) csupn
		return
	end if
	
	! Compute objective function
	
	call evalf(n,x,f)
	
	! Compute the subgradient
	
	call evals(n,x,s)
	
	! Compute the norm of s
	
	norm2s = norm2(s)
	
	! Set parameters Rconst, delta, opttol, sigma and beta
	
	Rconst = 1.0d0
	delta  = 0.5d0 * norm2s
	opttol = 1.0d-6
	optc   = 1.0d-5
	sigma  = 0.0d0
		
	theta  = 0.25d0
	lambda = 0.1d0 / 4.0d0
	gama   = 0.1d0 / 4.0d0
	
	nu     = ( 1.0d0 + 2.0d0 * gama ) / ( 1.0d0 - 2.0d0 * lambda )
	beta   = 2.0d0 / nu - opttol
		
	! Initialize parameters
	
	freckl     = f
	maxoutiter = 1.0d1 * n 
	
	! Compute optconv
	
	optconv = 1.0d-3 * ( 1.0d0 + abs( frec ) )
	
	! Counters
	
	outiter = 0
	l       = 0	
	lprev   = 0
	nfev    = 0
	ngev    = 0
		
	!---------------------------------------------------------------------  
	!     Main loop
	!---------------------------------------------------------------------     

	Main_loop: do
	
		! Check for simple decrease
		
		if ( outiter == 0 .or. f < frec ) then
				
				! Update frec and xrec 
				
				frec = f
				xrec = x
		end if
		
		! Compute optconv
	
		optconv = optc * ( 1.0d0 + abs( frec ) )
	
		! Print information

		if ( iprint ) then
	
			if ( outiter == 0 .or. ( mod(l,10) == 0 .and. l > 0 .and. l /= lprev ) ) write(*,1010)
			if ( outiter == 0 ) then
				write(*,1020) outiter,l,f,frec,delta,optconv
			elseif ( l /= lprev ) then
				write(*,1030) outiter,l,f,frec,flev,infoCG,delta,optconv
			end if
		end if
	
		!-------------------------------------------------------------------
		!     Test stopping criteria
		!-------------------------------------------------------------------
		
		! Test optimality
		
		if ( norm2s <= opttol ) then
		
				inform = 0
			
				! Stop timing
				
				call cpu_time(timefin)
				
				time = timefin - timeini
		
				if ( iprint ) write(*,1040) time
				
				return
		end if
		
		! Test convergence
		
		if ( delta <= optconv ) then
		
				inform = 1
				
				! Set x as the best iterate
				
				x = xrec

				! Stop timing
				
				call cpu_time(timefin)
				
				time = timefin - timeini
		
				if ( iprint ) write(*,1050) time
								
				return
		end if	
		
		! Test whether the number of iterations is exhausted
		
		if ( l >= maxoutiter ) then
				inform = 2
				
				! Stop timing
			
				call cpu_time(timefin)
			
				time = timefin - timeini
					
				if ( iprint ) write(*,1060) time
				
				return
		end if
		
		!-------------------------------------------------------------------
		!     Iteration
		!-------------------------------------------------------------------
		
		! Increment outiter
				
		outiter = outiter + 1
		
		! Check for sufficient decrease
		
		lprev = l
		
		if ( outiter > 1 .and. f <= freckl - 0.5d0 * delta ) then
		
				! Update l
		
				l = l + 1
		
				! Restart sigma
				
				sigma = zero
				
				! Update freckl
				
				freckl = frec
				
		elseif ( outiter > 1 .and. sigma > R ) then
				
				! Update l
		
				l = l + 1
				
				! Restart sigma
				
				sigma = zero
				
				! Update freckl
				
				freckl = frec
				
				! Decrease delta
				
				delta = 0.5d0 * delta 
				
				! Retrieve the iterate as xrec
				
				x = xrec
		end if
		
		! Update flev

		flev = freckl - delta
		
		! Compute the step size
		
		t = beta * ( f - flev ) / norm2s ** 2
		
		! Compute the aproximate projection and define the new iterate
				
		call CondG(n,x,x-t*s,w,m,lb,ub,equatn,linear,jcnnzmax,hnnzmax,&
                 gama,theta,lambda,erropt,itCG,infoCG)
                         
        x = w
		
		! Define the initial value of R = Rconst * ||x1 - x0||
		
		if ( outiter == 1 ) R = Rconst * norm2( x - xrec )
		
		! Update sigma
		
		sigma = sigma + norm2s * t
		
		! Compute objective function
	
		call evalf(n,x,f)
		
		! Compute the subgradient
		
		call evals(n,x,s)
		
		! Compute the norm of s
		
		norm2s = norm2(s)
		
			
	!---------------------------------------------------------------------
	!     Iterate
	!---------------------------------------------------------------------    

	end do Main_loop

	!--------------------------------------------------------------------- 
	!     End of main loop
	!---------------------------------------------------------------------
	
	! Non-executable statements
	
	1000 format(/,1X,'-------------------------------------------------------------------------------',/,1X,& 
					 '              Dynamic Subgradient Method with Inexact Projections              ',/,1X,& 
					 '-------------------------------------------------------------------------------',/,1X,& 
					 'Number of variables:',1X,I5)
	1001 format(/,1X,'Flag: the first iterate x must be feasible')	
	
	1010 format(/,2X,'out',5X,'l',4X,'f',11X,'frec',8X,'flev',6X,'CG',2X,'delta',5X,'optconv')
	1020 format(1X,I5,2X,I3,2(3X,1P,D9.2),6X,'-',8X,'-',2X,1P,D8.2,2X,1P,D8.2)
	1030 format(1X,I5,2X,I3,3(3X,1P,D9.2),3X,I1,2X,1P,D8.2,2X,1P,D8.2,2X,1P,D8.2)
	1040 format(/,1X,'Solution was found',/,1X, &
					 'Total CPU time in seconds: ',0P,F8.2)	
	1050 format(/,1X,'Convergence criterion satisfied: delta <= 10^(-3) * ( 1 + |frec| )',/,1X, &
					 'Total CPU time in seconds: ',0P,F8.2)					 				
	1060 format(/,1X,'Maximum of iterations reached',/,1X,&
					 'Total CPU time in seconds: ',0P,F8.2)
	1070 format(/,1X,'Flag: An error occurred while running the CondG method.',/,/,1X,&
				   	 'Total CPU time in seconds: ',0P,F8.2)	  					 

end subroutine dinamicsubg
