! ******************************************************************
! ******************************************************************

module myproblem

use globals

implicit none

  ! SUBROUTINES
  public :: inip,evalf,evals,evalc,evaljac,evalhc

contains

	subroutine inip(n,m,x,l,u,equatn,linear,jcnnzmax,hnnzmax,erropt)

	implicit none

	! SCALAR ARGUMENTS
	integer,intent(out) :: n,m,jcnnzmax,hnnzmax,erropt

	! ARRAY ARGUMENTS
	real(kind=8),allocatable,intent(out) :: x(:),l(:),u(:)
	logical,allocatable,intent(out) :: equatn(:),linear(:)

	! LOCAL SCALARS
	integer :: i,j,k,p,allocerr,nseed,LWORK,INFO,nsol
	real(kind=8) :: a,b,num
	real(kind=8),allocatable :: TAU(:),WORK(:),D(:),QWORK(:,:)
	logical,allocatable :: mk(:)
	
	! LOCAL ARRAYS
	integer,allocatable :: seed(:)
	
	! FUNCTIONS
	real(kind=8) :: dlaran

	! Number of variables

	n = 100
	
	! Number of constraints (except bound constraints) of each set
	
	m = 1
	
	! Set the error option for the projection using de Condg routine
	
	erropt = 1
	
	allocate(x(n),l(n),u(n),Q(n,n),r(n),equatn(m),linear(m),stat=allocerr)
	if ( allocerr .ne. 0 ) then
	   write(*,*) 'Allocation error in main program'
	   stop
	end if
	
	! Set the bounds constraints 
	
	do i = 1,n
		l(i) = 0.0d0
		u(i) = 1.0d20
	end do
	
	! Constraints: equality? linear?
	
	equatn(1) = .false.
	linear(1) = .false.
	
	
	! Upper bounds on the number of sparse-matrices non-null elements
	! jcnnzmax: number of non-null elements of the Jacobian
	! hnnzmax : number of non-null elements of the Hessians

	jcnnzmax = n
	hnnzmax  = n * ( n + 1 ) / 2
	
	! Generating the problem data
	
	LWORK = n**2
	
	call RANDOM_SEED(size = nseed)
	
	allocate(TAU(n),WORK(LWORK),seed(nseed))
	
	do i = 1,nseed
		seed(i) = 25 * i
	end do
	
	call RANDOM_SEED(PUT = seed)
	
	call RANDOM_NUMBER(Q)
	
	call DGEQRF( n, n, Q, n, TAU, WORK, LWORK, INFO )
	
	deallocate(WORK)
	allocate(WORK(n),D(n),mk(n),QWORK(n,n))

	call DORG2R( n, n, n, Q, n, TAU, WORK, INFO )	
	
	a = 1.0d1
	b = 1.0d3
	
	call RANDOM_NUMBER(WORK)
	
	WORK = a + WORK * ( b - a )
	
	a = 1.0d-6
	b = 1.0d-2
	
	call RANDOM_NUMBER(num)
	
	WORK(1) = a + num * ( b - a )
	
	mk(:) = .true.
	
	do i = 1,n
	   D(i) = MINVAL(WORK,mk)
	   mk(MINLOC(WORK,mk)) = .false.
	end do
	
	a = 0.8d0
	b = 0.99d0
	
	call RANDOM_NUMBER(num)
	
	num = a + num * ( b - a )
	
	r = ( num / sqrt(D(1)) ) * sign(1.0d0,Q(1,1)) * Q(:,1)
	r(n) = r(n) + 1.0d0 / sqrt(D(1))
	
	Q(:,1) = sign(1.0d0,Q(1,1)) * Q(:,1)
	do i = 1,n
		if ( Q(i,1) <= 0.0d0 ) then
			write(*,*)'Verificar decomposicao'
			return
		end if
	end do
	
	QWORK(:,:) = Q(:,:)
	
	Q(:,:) = 0.0d0
	do j = 1,n
				
		do k = 1,n
			Q(:,j) = Q(:,j) + D(k) * QWORK(j,k) * QWORK(:,k)
		end do
		
	end do
	
	! Initial points
	
	x = r
	
	x(:) = 0.0d0
	x(n) = 1.0d0 / sqrt(D(1))
	
	deallocate(TAU,WORK,D,QWORK,mk)

	end subroutine inip

	!***********************************************************************
	!***********************************************************************

	subroutine evalf(n,x,f)

	implicit none

	! SCALAR ARGUMENTS
	integer, intent(in) :: n
	real(kind=8), intent(out) :: f

	! ARRAY ARGUMENTS
	real(kind=8), intent(in) :: x(n)

	! LOCAL SCALARS
	integer :: i	

	f = 0.0d0
	do i = 1,n
			f = f + abs(x(i))
	end do	
					
	end subroutine evalf

	!***********************************************************************
	!***********************************************************************

	subroutine evals(n,x,s)

	implicit none

	! SCALAR ARGUMENTS
	integer, intent(in) :: n

	! ARRAY ARGUMENTS
	real(kind=8), intent(in)  :: x(n)
	real(kind=8), intent(out) :: s(n)

	! LOCAL SCALARS
	integer :: i
	real(kind=8) :: tol

	tol = 1.0d-6

	do i = 1,n
			if ( x(i) > tol ) then
					s(i) = 1.0d0
			elseif ( x(i) < - tol ) then
					s(i) = - 1.0d0
			else
					s(i) = 0.0d0
			end if
	end do

	end subroutine evals

	! ******************************************************************
	! ******************************************************************

	subroutine evalc(n,x,ind,c,flag)

	implicit none

	! SCALAR ARGUMENTS
	integer,      intent(in)  :: n,ind
	integer,      intent(out) :: flag
	real(kind=8), intent(out) :: c

	! ARRAY ARGUMENTS
	real(kind=8), intent(in)  :: x(n)
	
	real(kind=8),allocatable :: Qx(:)
	
	flag = 0

	allocate(Qx(n))

	Qx = matmul(Q,x)
	
	c = dot_product(x,Qx) - 2.0d0 * dot_product(r,Qx) +       &
		dot_product(r,matmul(Q,r)) - 1.0d0
		
	deallocate(Qx)

	end subroutine evalc

	! ******************************************************************
	! ******************************************************************

	subroutine evaljac(n,x,ind,jcvar,jcval,jcnnz,lim,lmem,flag)

	implicit none

	! SCALAR ARGUMENTS
	logical, intent(out) :: lmem
	integer, intent(in)  :: ind,lim,n
	integer, intent(out) :: flag,jcnnz

	! ARRAY ARGUMENTS
	integer,      intent(out) :: jcvar(lim)
	real(kind=8), intent(in)  :: x(n)
	real(kind=8), intent(out) :: jcval(lim)
	
	! LOCAL SCALARS
	integer :: i

	flag = 0
	lmem = .false.

	jcnnz = n

	if ( jcnnz .gt. lim ) then
		lmem = .true.
		return
	end if
		  
	do i = 1,n
		jcvar(i) = i
		jcval(i) = 2.0d0 * dot_product(Q(i,:),x-r) 
	end do
	
	end subroutine evaljac

	! ******************************************************************
	! ******************************************************************

	subroutine evalhc(n,x,ind,hcrow,hccol,hcval,hcnnz,lim,lmem,flag)

	implicit none

	! SCALAR ARGUMENTS
	logical, intent(out) :: lmem
	integer, intent(in)  :: ind,lim,n
	integer, intent(out) :: flag,hcnnz

	! ARRAY ARGUMENTS
	integer,      intent(out) :: hccol(lim),hcrow(lim)
	real(kind=8), intent(in)  :: x(n)
	real(kind=8), intent(out) :: hcval(lim)

!	! LOCAL SCALARS
	integer :: i,j,k

	flag = 0
	lmem = .false.

	hcnnz = n * ( n + 1 ) / 2

	k = 0
	do i = 1,n
		do j = 1,i
			k = k + 1
			hcrow(k) = i
			hccol(k) = j 
			hcval(k) = 2.0d0 * Q(i,j)
		end do
	end do
		  
	end subroutine evalhc	

end module myproblem
